@@ -1,32 +1,72 @@
+import enum
 import os
 from pathlib import Path
 import random
-from yarl import URL
+import re
 from typing import (
-    Any, Callable, Iterable, Union,
-    List, Tuple, Sequence,
+    Callable,
+    Iterable,
+    List,
+    Mapping,
+    Optional,
+    Sequence,
+    Tuple,
+    TypeVar,
+    Union,
+    cast,
 )
 
 import appdirs
+from yarl import URL
 
 __all__ = [
+    'parse_api_version',
     'get_config',
     'set_config',
     'APIConfig',
+    'API_VERSION',
+    'DEFAULT_CHUNK_SIZE',
+    'MAX_INFLIGHT_CHUNKS',
 ]
 
-_config = None
-_undefined = object()
 
-DEFAULT_CHUNK_SIZE = 256 * 1024  # 256 KiB
+class Undefined(enum.Enum):
+    token = object()
+
+
+_config = None
+_undefined = Undefined.token
+
+API_VERSION = (6, '20200815')
+
+DEFAULT_CHUNK_SIZE = 16 * (2**20)  # 16 MiB
+MAX_INFLIGHT_CHUNKS = 4
 
 local_state_path = Path(appdirs.user_state_dir('backend.ai', 'Lablup'))
 local_cache_path = Path(appdirs.user_cache_dir('backend.ai', 'Lablup'))
 
 
-def get_env(key: str, default: Any = _undefined, *,
-            clean: Callable[[str], Any] = lambda v: v):
-    '''
+def parse_api_version(value: str) -> Tuple[int, str]:
+    match = re.search(r'^v(?P<major>\d+)\.(?P<date>\d{8})$', value)
+    if match is not None:
+        return int(match.group(1)), match.group(2)
+    raise ValueError('Could not parse the given API version string', value)
+
+
+T = TypeVar('T')
+
+
+def default_clean(v: str) -> T:
+    return cast(T, v)
+
+
+def get_env(
+    key: str,
+    default: Union[str, Undefined] = _undefined,
+    *,
+    clean: Callable[[str], T] = default_clean,
+) -> T:
+    """
     Retrieves a configuration value from the environment variables.
     The given *key* is uppercased and prefixed by ``"BACKEND_"`` and then
     ``"SORNA_"`` if the former does not exist.
@@ -39,16 +79,16 @@ def get_env(key: str, default: Any = _undefined, *,
         The default is returning the value as-is.
 
     :returns: The value processed by the *clean* function.
-    '''
+    """
     key = key.upper()
-    v = os.environ.get('BACKEND_' + key)
-    if v is None:
-        v = os.environ.get('SORNA_' + key)
-    if v is None:
+    raw = os.environ.get('BACKEND_' + key)
+    if raw is None:
+        raw = os.environ.get('SORNA_' + key)
+    if raw is None:
         if default is _undefined:
             raise KeyError(key)
-        v = default
-    return clean(v)
+        raw = default
+    return clean(raw)
 
 
 def bool_env(v: str) -> bool:
@@ -60,11 +100,11 @@ def bool_env(v: str) -> bool:
     raise ValueError('Unrecognized value of boolean environment variable', v)
 
 
-def _clean_urls(v: str) -> List[URL]:
+def _clean_urls(v: Union[URL, str]) -> List[URL]:
     if isinstance(v, URL):
         return [v]
+    urls = []
     if isinstance(v, str):
-        urls = []
         for entry in v.split(','):
             url = URL(entry)
             if not url.is_absolute():
@@ -73,16 +113,14 @@ def _clean_urls(v: str) -> List[URL]:
     return urls
 
 
-def _clean_tokens(v):
-    if isinstance(v, str):
-        if not v:
-            return tuple()
-        return tuple(v.split(','))
-    return tuple(iter(v))
+def _clean_tokens(v: str) -> Tuple[str, ...]:
+    if not v:
+        return tuple()
+    return tuple(v.split(','))
 
 
 class APIConfig:
-    '''
+    """
     Represents a set of API client configurations.
     The access key and secret key are mandatory -- they must be set in either
     environment variables or as the explicit arguments.
@@ -116,68 +154,81 @@ class APIConfig:
         access key) to be automatically mounted upon any
         :func:`Kernel.get_or_create()
         <ai.backend.client.kernel.Kernel.get_or_create>` calls.
-    '''
+    """
 
-    DEFAULTS = {
+    DEFAULTS: Mapping[str, str] = {
         'endpoint': 'https://api.backend.ai',
         'endpoint_type': 'api',
-        'version': 'v4.20190615',
+        'version': f'v{API_VERSION[0]}.{API_VERSION[1]}',
         'hash_type': 'sha256',
         'domain': 'default',
         'group': 'default',
-        'connection_timeout': 10.0,
-        'read_timeout': None,
+        'connection_timeout': '10.0',
+        'read_timeout': '0',
     }
-    '''
-    The default values except the access and secret keys.
-    '''
+    """
+    The default values for config parameterse settable via environment variables
+    xcept the access and secret keys.
+    """
 
-    def __init__(self, *,
-                 endpoint: Union[URL, str] = None,
-                 endpoint_type: str = None,
-                 domain: str = None,
-                 group: str = None,
-                 version: str = None,
-                 user_agent: str = None,
-                 access_key: str = None,
-                 secret_key: str = None,
-                 hash_type: str = None,
-                 vfolder_mounts: Iterable[str] = None,
-                 skip_sslcert_validation: bool = None,
-                 connection_timeout: float = None,
-                 read_timeout: float = None) -> None:
-        from . import get_user_agent  # noqa; to avoid circular imports
+    _endpoints: List[URL]
+    _group: str
+    _hash_type: str
+
+    def __init__(
+        self, *,
+        endpoint: Union[URL, str] = None,
+        endpoint_type: str = None,
+        domain: str = None,
+        group: str = None,
+        version: str = None,
+        user_agent: str = None,
+        access_key: str = None,
+        secret_key: str = None,
+        hash_type: str = None,
+        vfolder_mounts: Iterable[str] = None,
+        skip_sslcert_validation: bool = None,
+        connection_timeout: float = None,
+        read_timeout: float = None,
+        announcement_handler: Callable[[str], None] = None,
+    ) -> None:
+        from . import get_user_agent
         self._endpoints = (
             _clean_urls(endpoint) if endpoint else
-            get_env('ENDPOINT', self.DEFAULTS['endpoint'], clean=_clean_urls))
+            get_env('ENDPOINT', self.DEFAULTS['endpoint'], clean=_clean_urls)
+        )
         random.shuffle(self._endpoints)
-        self._endpoint_type = endpoint_type if endpoint_type \
-                              else get_env('ENDPOINT_TYPE', self.DEFAULTS['endpoint_type'])
-        self._domain = domain if domain else get_env('DOMAIN', self.DEFAULTS['domain'])
-        self._group = group if group else get_env('GROUP', self.DEFAULTS['group'])
-        self._version = version if version else self.DEFAULTS['version']
-        self._user_agent = user_agent if user_agent else get_user_agent()
+        self._endpoint_type = endpoint_type if endpoint_type is not None else \
+            get_env('ENDPOINT_TYPE', self.DEFAULTS['endpoint_type'], clean=str)
+        self._domain = domain if domain is not None else \
+            get_env('DOMAIN', self.DEFAULTS['domain'], clean=str)
+        self._group = group if group is not None else \
+            get_env('GROUP', self.DEFAULTS['group'], clean=str)
+        self._version = version if version is not None else \
+            self.DEFAULTS['version']
+        self._user_agent = user_agent if user_agent is not None else get_user_agent()
         if self._endpoint_type == 'api':
-            self._access_key = access_key if access_key is not None \
-                               else get_env('ACCESS_KEY', '')
-            self._secret_key = secret_key if secret_key is not None \
-                               else get_env('SECRET_KEY', '')
+            self._access_key = access_key if access_key is not None else \
+                get_env('ACCESS_KEY', '')
+            self._secret_key = secret_key if secret_key is not None else \
+                get_env('SECRET_KEY', '')
         else:
             self._access_key = 'dummy'
             self._secret_key = 'dummy'
-        self._hash_type = hash_type.lower() if hash_type else \
-                          self.DEFAULTS['hash_type']
+        self._hash_type = hash_type.lower() if hash_type is not None else \
+                          cast(str, self.DEFAULTS['hash_type'])
         arg_vfolders = set(vfolder_mounts) if vfolder_mounts else set()
-        env_vfolders = set(get_env('VFOLDER_MOUNTS', [], clean=_clean_tokens))
+        env_vfolders = set(get_env('VFOLDER_MOUNTS', '', clean=_clean_tokens))
         self._vfolder_mounts = [*(arg_vfolders | env_vfolders)]
         # prefer the argument flag and fallback to env if the flag is not set.
         self._skip_sslcert_validation = (skip_sslcert_validation
              if skip_sslcert_validation else
              get_env('SKIP_SSLCERT_VALIDATION', 'no', clean=bool_env))
         self._connection_timeout = connection_timeout if connection_timeout else \
-            get_env('CONNECTION_TIMEOUT', self.DEFAULTS['connection_timeout'])
+            get_env('CONNECTION_TIMEOUT', self.DEFAULTS['connection_timeout'], clean=float)
         self._read_timeout = read_timeout if read_timeout else \
-            get_env('READ_TIMEOUT', self.DEFAULTS['read_timeout'])
+            get_env('READ_TIMEOUT', self.DEFAULTS['read_timeout'], clean=float)
+        self._announcement_handler = announcement_handler
 
     @property
     def is_anonymous(self) -> bool:
@@ -185,16 +236,16 @@ class APIConfig:
 
     @property
     def endpoint(self) -> URL:
-        '''
+        """
         The currently active endpoint URL.
         This may change if there are multiple configured endpoints
         and the current one is not accessible.
-        '''
+        """
         return self._endpoints[0]
 
     @property
     def endpoints(self) -> Sequence[URL]:
-        '''All configured endpoint URLs.'''
+        """All configured endpoint URLs."""
         return self._endpoints
 
     def rotate_endpoints(self):
@@ -202,76 +253,84 @@ class APIConfig:
             item = self._endpoints.pop(0)
             self._endpoints.append(item)
 
+    def load_balance_endpoints(self):
+        pass
+
     @property
     def endpoint_type(self) -> str:
-        '''
+        """
         The configured endpoint type.
-        '''
+        """
         return self._endpoint_type
 
     @property
     def domain(self) -> str:
-        '''The configured domain.'''
+        """The configured domain."""
         return self._domain
 
     @property
     def group(self) -> str:
-        '''The configured group.'''
+        """The configured group."""
         return self._group
 
     @property
     def user_agent(self) -> str:
-        '''The configured user agent string.'''
+        """The configured user agent string."""
         return self._user_agent
 
     @property
     def access_key(self) -> str:
-        '''The configured API access key.'''
+        """The configured API access key."""
         return self._access_key
 
     @property
     def secret_key(self) -> str:
-        '''The configured API secret key.'''
+        """The configured API secret key."""
         return self._secret_key
 
     @property
     def version(self) -> str:
-        '''The configured API protocol version.'''
+        """The configured API protocol version."""
         return self._version
 
     @property
     def hash_type(self) -> str:
-        '''The configured hash algorithm for API authentication signatures.'''
+        """The configured hash algorithm for API authentication signatures."""
         return self._hash_type
 
     @property
-    def vfolder_mounts(self) -> Tuple[str, ...]:
-        '''The configured auto-mounted vfolder list.'''
+    def vfolder_mounts(self) -> Sequence[str]:
+        """The configured auto-mounted vfolder list."""
         return self._vfolder_mounts
 
     @property
     def skip_sslcert_validation(self) -> bool:
-        '''Whether to skip SSL certificate validation for the API gateway.'''
+        """Whether to skip SSL certificate validation for the API gateway."""
         return self._skip_sslcert_validation
 
     @property
     def connection_timeout(self) -> float:
-        '''The maximum allowed duration for making TCP connections to the server.'''
+        """The maximum allowed duration for making TCP connections to the server."""
         return self._connection_timeout
 
     @property
     def read_timeout(self) -> float:
-        '''The maximum allowed waiting time for the first byte of the response from the server.'''
+        """The maximum allowed waiting time for the first byte of the response from the server."""
         return self._read_timeout
 
+    @property
+    def announcement_handler(self) -> Optional[Callable[[str], None]]:
+        '''The announcement handler to display server-set announcements.'''
+        return self._announcement_handler
+
 
 def get_config():
-    '''
+    """
     Returns the configuration for the current process.
     If there is no explicitly set :class:`APIConfig` instance,
     it will generate a new one from the current environment variables
     and defaults.
-    '''
+    """
     global _config
     if _config is None:
         _config = APIConfig()
@@ -279,8 +338,8 @@ def get_config():
 
 
 def set_config(conf: APIConfig):
-    '''
+    """
     Sets the configuration used throughout the current process.
-    '''
+    """
     global _config
     _config = conf
